@@ -33,6 +33,51 @@
 #'   * `length` returns number of elements in `x`
 #' @method format pt2mod
 #' @rdname s3methods
+#' @examples
+#' ## First create some ProTrackR2 objects to which
+#' ## S3 methods can be applied:
+#' 
+#' mod      <- pt2_read_mod(pt2_demo())
+#' patterns <- mod$patterns
+#' pattern  <- patterns[[1]]
+#' cells    <- pattern[1:4,1]
+#' cell     <- cells[[1]]
+#' samples  <- mod$samples
+#' sample   <- samples[[1]]
+#' cmnd     <- pt2_command(cell)
+#' 
+#' ## Let's go wild with S3 methods:
+#' print(mod)
+#' print(patterns)
+#' print(pattern)
+#' print(cells)
+#' print(cell)
+#' print(samples)
+#' print(sample)
+#' print(cmnd)
+#' 
+#' format(mod)
+#' format(patterns)
+#' format(pattern)
+#' format(cells)
+#' format(cell)
+#' format(samples)
+#' format(sample)
+#' format(cmnd)
+#' 
+#' as.character(pattern)
+#' as.character(cells)
+#' as.character(cell)
+#' as.character(cmnd)
+#' 
+#' as.raw(mod)
+#' as.raw(pattern)
+#' as.raw(cells)
+#' as.raw(cell)
+#' as.raw(sample)
+#' as.raw(cmnd)
+#' 
+#' as.integer(sample)
 #' @export
 format.pt2mod <- function(x, ...) {
   dur <- pt2_duration(x) |>
@@ -56,19 +101,26 @@ format.pt2pat <- function(x, padding = " ", empty_char = "-", fmt = getOption("p
     matrix(ncol = 4, byrow = T)
 }
 
+#' @method as.character pt2command
+#' @rdname s3methods
+#' @export
+as.character.pt2command <- function(x, ...) {
+  result <- character()
+  for (i in seq_len(length(x))) {
+    cmnd <- as.integer(unclass(x[[i]]))
+    cmnd <- sprintf("%X%02X", cmnd[[1]], cmnd[[2]])
+    result <- c(result, cmnd)
+  }
+  result
+}
+
 #' @method format pt2command
 #' @rdname s3methods
 #' @export
 format.pt2command <- function(x, fmt = getOption("pt2_effect_format"), ...) {
-  if (typeof(x) == "raw") {
-    matrix(x, ncol = 2L, byrow = TRUE) |>
-      apply(1, .command_format, fmt, simplify = FALSE) |>
-      unlist()
-  } else if (inherits(x, "pt2celllist")) {
-    lapply(x, .command_format, fmt) |> unlist()
-  } else {
-    .command_format(x, fmt)
-  }
+  matrix(x, ncol = 2L, byrow = TRUE) |>
+    apply(1, .command_format, fmt, simplify = FALSE) |>
+    unlist()
 }
 
 #' @method print pt2command
@@ -134,22 +186,17 @@ as.raw.pt2celllist <- function(x, ...) {
 #' @rdname s3methods
 #' @export
 as.raw.pt2celllist.logical <- function(x, compact = TRUE, ...) {
-  if (typeof(x) == "raw") {
-    cur_notation <- attributes(x)$compact_notation
-    width <- ifelse(cur_notation, 4, 6)
-    x <-
-      matrix(unclass(x), ncol = width, byrow = TRUE) |>
-      apply(1, \(y) {
-        class(y) <- "pt2cell"
-        attributes(y)$compact_notation <- cur_notation
-        as.raw.pt2cell(y, compact = compact)
-      }, simplify = FALSE) |> unlist()
-  } else {
-    x <- lapply(x, \(y) as.raw.pt2cell(y, compact = compact, ...)) |> unlist()
-  }
-  class(x) <- "pt2celllist"
-  attributes(x)$compact_notation <- compact
-  x
+  d <- attr(x, "celldim")
+  cur_notation <- attributes(x)$compact_notation
+  width <- ifelse(cur_notation, 4L, pt_cell_bytesize())
+  x <-
+    matrix(unclass(x), ncol = width, byrow = TRUE) |>
+    apply(1, \(y) {
+      class(y) <- "pt2cell"
+      attributes(y)$compact_notation <- cur_notation
+      as.raw.pt2cell(y, compact = compact)
+    }, simplify = FALSE) |> unlist()
+  structure(x, class = "pt2celllist", celldim = d, compact_notation = compact)
 }
 
 #' @method as.raw pt2pat
@@ -168,16 +215,12 @@ as.raw.pt2pat.logical <- function(x, compact = TRUE, ...) {
   if (typeof(x) == "raw") {
     cur_notation <- attributes(x)$compact_notation
     if (cur_notation == compact) return (x)
-    if (cur_notation) {
-      x <- pt_decode_compact_cell(x)
-    } else {
-      x <- pt_encode_compact_cell(x)
-    }
+    x <- pt_decode_compact_cell(x)
     class(x) <- "pt2pat"
     attributes(x)$compact_notation <- !cur_notation
     x
   } else {
-    cells_as_raw_(x$mod, as.integer(x$i), compact, TRUE, 0L, 0L)
+    cells_as_raw_(x$mod, as.integer(x$i), compact, 0L, 0L)
   }
 }
 
@@ -201,25 +244,26 @@ print.pt2cell <- function(x, ...) {
 as.character.pt2cell <- function(x, ...) {
   x <- format(x, ...)
   if (requireNamespace("cli", quietly = TRUE))
-    x <- apply(x, 2, cli::ansi_strip, simplify = TRUE)
+    x <- cli::ansi_strip(x)
   x
 }
 
+#' @method as.character pt2celllist
+#' @rdname s3methods
+#' @export
+as.character.pt2celllist <- function(x, ...) {
+  result <- character()
+  for (i in seq_len(length(x))) {
+    result <- c(result, as.character(x[[i]]))
+  }
+  result
+}
+
 #' @method as.raw pt2command
 #' @rdname s3methods
 #' @export
 as.raw.pt2command <- function(x, ...) {
-  if (typeof(x) == "raw") return(x)
-
-  if (inherits(x, "pt2celllist") || is.null(names(x))) {
-    mods <- lapply(x, `[[`, "mod")
-    i <- lapply(x, `[[`, "i") |> unlist()
-    j <- lapply(x, `[[`, "j") |> unlist()
-    k <- lapply(x, `[[`, "k") |> unlist()
-    pt_eff_command_(mods, i, k, j)
-  } else {
-    pt_eff_command_(list(x$mod), x$i, x$k, x$j)
-  }
+  x # pt2command is already raw
 }
 
 #' @method as.raw pt2cell
@@ -235,37 +279,25 @@ as.raw.pt2cell <- function(x, ...) {
 #' @rdname s3methods
 #' @export
 as.raw.pt2cell.logical <- function(x, compact = TRUE, ...) {
-  if (typeof(x) == "raw") {
-    cur_notation <- attributes(x)$compact_notation
-    if (is.null(cur_notation))
-      stop("Unknown notation of `pt2cell`")
-    if (cur_notation == compact) return (x)
-    if (cur_notation) {
-      x <- pt_decode_compact_cell(x)
-    } else {
-      x <- pt_encode_compact_cell(x)
-    }
-    class(x) <- "pt2cell"
-    attributes(x)$compact_notation <- !cur_notation
-    x
+  cur_notation <- attributes(x)$compact_notation
+  if (is.null(cur_notation))
+    stop("Unknown notation of `pt2cell`")
+  if (cur_notation == compact) return (x)
+  if (cur_notation) {
+    x <- pt_decode_compact_cell(x)
   } else {
-    result <-
-      cells_as_raw_(x$mod, as.integer(x$i), compact, FALSE,
-                    as.integer(x$j), as.integer(x$k))
-    class(result) <- "pt2cell"
-    result
+    x <- pt_encode_compact_cell(x)
   }
+  class(x) <- "pt2cell"
+  attributes(x)$compact_notation <- !cur_notation
+  x
 }
 
 #' @method format pt2samp
 #' @rdname s3methods
 #' @export
 format.pt2samp <- function(x, ...) {
-  si <- if (typeof(x) == "raw") {
-    attributes(x)$sample_info
-  } else {
-    mod_sample_info_(x$mod, as.integer(x$i))
-  }
+  si <- attributes(x)$sample_info
   sprintf("PT2 Sample '%s' [%i]", si$text, si$length)
 }
 
@@ -337,17 +369,13 @@ as.raw.pt2samp <- function(x, ...) {
 #' @rdname s3methods
 #' @export
 as.integer.pt2samp <- function(x, ...) {
-  if (typeof(x) == "raw") {
-    a <- attributes(x)
-    x <-
-      unclass(x) |>
-      as.integer()
-    x[x > 127L] <- x[x > 127L] - 256L
-    attributes(x) <- a
-    x
-  } else {
-    mod_sample_as_int_(x$mod, x$i)
-  }
+  a <- attributes(x)
+  x <-
+    unclass(x) |>
+    as.integer()
+  x[x > 127L] <- x[x > 127L] - 256L
+  attributes(x) <- a[!names(a) %in% "class"]
+  x
 }
 
 .command_format <- function(x, fmt = getOption("pt2_effect_format")) {
@@ -387,7 +415,7 @@ as.integer.pt2samp <- function(x, ...) {
 #' @rdname s3methods
 #' @export
 length.pt2celllist <- function(x, ...) {
-  fct  <- ifelse(attributes(x)$compact_notation, 4L, 6L)
+  fct  <- ifelse(attributes(x)$compact_notation, 4L, pt_cell_bytesize())
   fct  <- ifelse(typeof(x) == "raw", fct, 1L)
   length(unclass(x))/fct
 }
@@ -398,4 +426,4 @@ length.pt2celllist <- function(x, ...) {
 length.pt2command <- function(x, ...) {
   fct  <- ifelse(typeof(x) == "raw", 2L, 1L)
   length(unclass(x))/fct
-}
\ No newline at end of file
+}
