@@ -1,14 +1,11 @@
 #include <cpp11.hpp>
-#include <vector>
 #include "get_mod.h"
 #include "mod_header.h"
-#include "pt2-clone/pt2_module_loader.h"
-#include "pt2-clone/pt2_module_saver.h"
-#include "pt2-clone/pt2_replayer_light.h"
+#include "pt2-clone.h"
 using namespace cpp11;
 
 SEXP create_mod_pointer(module_t * my_song) {
-  if (my_song == NULL) Rf_error("Failed to create module");
+  if (my_song == NULL) stop("Failed to create module");
   external_pointer<module_t, modFree2>mod(my_song);
   sexp result = as_sexp(mod);
   result.attr("class") = "pt2mod";
@@ -24,7 +21,7 @@ SEXP open_mod_(raws data) {
 }
 
 [[cpp11::register]]
-SEXP mod_as_raw_(SEXP mod) {
+raws mod_as_raw_(SEXP mod) {
   module_t *my_song = get_mod(mod);
   int n_pat = n_patterns_internal(mod);
   int mod_header = 154;
@@ -36,17 +33,18 @@ SEXP mod_as_raw_(SEXP mod) {
   }
   int mod_size = mod_header + MOD_SAMPLES * sample_header +
     n_pat * pattern_size + sample_data;
-  std::vector<uint8_t> buff(mod_size);
-  uint8_t * buffer = buff.data();
-  uint32_t n_written = modSave2(my_song, buffer, mod_size);
+  writable::raws buf((R_xlen_t)mod_size);
+  uint8_t * rbuf = (uint8_t *)RAW(buf.data());
 
-  if (n_written != (uint32_t)mod_size) Rf_error("Unexpected data size");
-  
-  return Rf_coerceVector(as_sexp(buff), RAWSXP);
+  uint32_t n_written = modSave2(my_song, rbuf, mod_size);
+
+  if (n_written != (uint32_t)mod_size) stop("Unexpected data size");
+
+  return buf;
 }
 
 [[cpp11::register]]
-SEXP new_mod_(strings name) {
+SEXP new_mod_(std::string name) {
   module_t * my_song = createEmptyMod();
   set_mod_name_internal(my_song, name);
   return create_mod_pointer(my_song);
