@@ -1,6 +1,5 @@
 #include <cpp11.hpp>
-#include "pt2-clone/pt2_config.h"
-#include "pt2-clone/pt2_audio.h"
+#include "pt2-clone.h"
 using namespace cpp11;
 
 [[cpp11::register]]
